<?php

#
# ss_cisco_5min_mem_stats.php
# version 0.1
# February 19, 2013
#
# Author: Chadd J. Christiansen
# Based on: ss_cisco_cpu_usage.php by Eric A. Hall
#
# This software is licensed under the same terms as the Cacti project
#

#
# load the Cacti configuration settings if they aren't already present
#
if (isset($config) == FALSE) {

	if (file_exists(dirname(__FILE__) . "/../include/config.php")) {
		include_once(dirname(__FILE__) . "/../include/config.php");
	}

	if (file_exists(dirname(__FILE__) . "/../include/global.php")) {
		include_once(dirname(__FILE__) . "/../include/global.php");
	}

	if (isset($config) == FALSE) {
		echo ("FATAL: Unable to load Cacti configuration files \n");
		return;
	}
}

#
# load the Cacti SNMP libraries if they aren't already present
#
if (defined('SNMP_METHOD_PHP') == FALSE) {

	if (file_exists(dirname(__FILE__) . "/../lib/snmp.php")) {
		include_once(dirname(__FILE__) . "/../lib/snmp.php");
	}

	if (defined('SNMP_METHOD_PHP') == FALSE) {
		echo ("FATAL: Unable to load SNMP libraries \n");
		return;
	}
}

#
# call the main function manually if executed outside the Cacti script server
#
if (isset($GLOBALS['called_by_script_server']) == FALSE) {

	array_shift($_SERVER["argv"]);
	print call_user_func_array("ss_cisco_5min_mem_stats", $_SERVER["argv"]);
}

#
# main function
#
function ss_cisco_5min_mem_stats($protocol_bundle="",
	$cacti_request="", $data_request="", $data_request_key="") {

	#
	# 1st function argument contains the protocol-specific bundle
	#
	# use '===' matching for strpos in case colon is 1st character
	#
	if ((trim($protocol_bundle) == "") || (strpos($protocol_bundle, ":") === FALSE)) {

		echo ("FATAL: No SNMP parameter bundle provided\n");
		ss_cisco_5min_mem_stats_syntax();
		return;
	}

	$protocol_array = explode(":", $protocol_bundle);

	if (count($protocol_array) < 11) {

		echo ("FATAL: Not enough elements in SNMP parameter bundle\n");
		ss_cisco_5min_mem_stats_syntax();
		return;
	}

	if (count($protocol_array) > 11) {

		echo ("FATAL: Too many elements in SNMP parameter bundle\n");
		ss_cisco_5min_mem_stats_syntax();
		return;
	}

	#
	# 1st bundle element is $snmp_hostname
	#
	$snmp_hostname = trim($protocol_array[0]);

	if ($snmp_hostname == "") {

		echo ("FATAL: Hostname not specified in SNMP parameter bundle\n");
		ss_cisco_5min_mem_stats_syntax();
		return;
	}

	#
	# 2nd bundle element is $snmp_version
	#
	$snmp_version = trim($protocol_array[1]);

	if ($snmp_version == "") {

		echo ("FATAL: SNMP version not specified in SNMP parameter bundle\n");
		ss_cisco_5min_mem_stats_syntax();
		return;
	}

	if (($snmp_version != 1) and ($snmp_version != 2) and ($snmp_version != 3)) {

		echo ("FATAL: \"$snmp_version\" is not a valid SNMP version\n");
		ss_cisco_5min_mem_stats_syntax();
		return;
	}

	#
	# 3rd bundle element is $snmp_community
	#
	$snmp_community = trim($protocol_array[2]);

	if (($snmp_version != 3) and ($snmp_community == "")) {

		echo ("FATAL: SNMP v$snmp_version community not specified in SNMP parameter bundle\n");
		ss_cisco_5min_mem_stats_syntax();
		return;
	}

	#
	# 4th bundle element is $snmp_v3_username
	#
	$snmp_v3_username = trim($protocol_array[3]);

	#
	# 5th bundle element is $snmp_v3_password
	#
	$snmp_v3_password = trim($protocol_array[4]);

	#
	# 6th bundle element is $snmp_v3_authproto
	#
	$snmp_v3_authproto = trim($protocol_array[5]);

	#
	# 7th bundle element is $snmp_v3_privpass
	#
	$snmp_v3_privpass = trim($protocol_array[6]);

	#
	# 8th bundle element is $snmp_v3_privproto
	#
	$snmp_v3_privproto = trim($protocol_array[7]);

	#
	# 9th bundle element is $snmp_v3_context
	#
	$snmp_v3_context = trim($protocol_array[8]);

	#
	# 10th bundle element is $snmp_port
	#
	$snmp_port = trim($protocol_array[9]);

	if ($snmp_port == "") {

		#
		# if the value was omitted use the default port number
		#
		$snmp_port = 161;
	}

	if (is_numeric($snmp_port) == FALSE) {

		echo ("FATAL: Non-numeric SNMP port \"$snmp_port\" specified in SNMP parameter bundle\n");
		ss_cisco_5min_mem_stats_syntax();
		return;
	}

	#
	# 11th bundle element is $snmp_timeout
	#
	$snmp_timeout = trim($protocol_array[10]);

	if ($snmp_timeout == "") {

		#
		# if the value was omitted use the global default timeout
		#
		$snmp_timeout = read_config_option("snmp_timeout");
	}

	if (is_numeric($snmp_timeout) == FALSE) {

		echo ("FATAL: Non-numeric SNMP timeout \"$snmp_timeout\" specified in SNMP parameter bundle\n");
		ss_cisco_5min_mem_stats_syntax();
		return;
	}

	#
	# these aren't parameters, but go ahead and out $snmp_retries and $snmp_maxoids
	# from the global settings
	#
	$snmp_retries = read_config_option("snmp_retries");
	$snmp_maxoids = read_config_option("max_get_size");

	#
	# 2nd function argument is $cacti_request
	#
	$cacti_request = strtolower(trim($cacti_request));

	if ($cacti_request == "") {

		echo ("FATAL: No Cacti request provided\n");
		ss_cisco_5min_mem_stats_syntax();
		return;
	}

	if (($cacti_request != "index") &&
		($cacti_request != "query") &&
		($cacti_request != "get")) {

		echo ("FATAL: \"$cacti_request\" is not a valid Cacti request\n");
		ss_cisco_5min_mem_stats_syntax();
		return;
	}

	#
	# remaining function arguments are $data_request and $data_request_key
	#
	if (($cacti_request == "query") || ($cacti_request == "get")) {

		$data_request = strtolower(trim($data_request));

		if ($data_request == "") {

			echo ("FATAL: No data requested for Cacti \"$cacti_request\" request\n");
			ss_cisco_5min_mem_stats_syntax();
			return;
		}

		if (($data_request != "mempool") &&
			($data_request != "mempoolname") &&
			($data_request != "mempooltotal") &&
			($data_request != "mempoolused")) {

			echo ("FATAL: \"$data_request\" is not a valid data request\n");
			ss_cisco_5min_mem_stats_syntax();
			return;
		}

		#
		# get the index variable
		#
		if ($cacti_request == "get") {

			$data_request_key = strtolower(trim($data_request_key));

			if ($data_request_key == "") {

				echo ("FATAL: No index value provided for \"$data_request\" data request\n");
				ss_cisco_5min_mem_stats_syntax();
				return;
			}
		}

		#
		# clear out spurious command-line parameters on query requests
		#
		else {
			$data_request_key = "";
		}
	}

	#
	# build a nested array of data elements for future use
	#
	$oid_array = array ("memPoolindex" => ".1.3.6.1.4.1.9.9.48.1.1.1.2",
		"memPoolused" => ".1.3.6.1.4.1.9.9.48.1.1.1.5",
		"memPoolfree" => ".1.3.6.1.4.1.9.9.48.1.1.1.6");

	#
	# build the snmp_arguments array for future use
	#
	# note that the array structure varies according to the version of Cacti in use
	#
	if (isset($GLOBALS['config']['cacti_version']) == FALSE) {

		echo ("FATAL: Unable to determine Cacti version\n");
		return;
	}

	elseif (substr($GLOBALS['config']['cacti_version'],0,5) == "0.8.6") {

		$snmp_arguments = array(
			$snmp_hostname,
			$snmp_community,
			"",
			$snmp_version,
			$snmp_v3_username,
			$snmp_v3_password,
			$snmp_port,
			$snmp_timeout);

		#
		# Cacti 0.8.6 SNMP timeout used milliseconds, while PHP uses Net-SNMP foormat, which
		# is typically microseconds. Normalize by multiplying the timeout value by 1000.
		#
		$snmp_timeout = ($snmp_timeout * 1000);
	}

	elseif (substr($GLOBALS['config']['cacti_version'],0,5) >= "0.8.7") {

		$snmp_arguments = array(
			$snmp_hostname,
			$snmp_community,
			"",
			$snmp_version,
			$snmp_v3_username,
			$snmp_v3_password,
			$snmp_v3_authproto,
			$snmp_v3_privpass,
			$snmp_v3_privproto,
			$snmp_v3_context,
			$snmp_port,
			$snmp_timeout,
			$snmp_retries,
			$snmp_maxoids);
	}

	else {
		echo ("FATAL: \"" . $GLOBALS['config']['cacti_version'] .
			"\" is not a supported Cacti version\n");
		return;
	}

	#
	# if they want data for just one mempool, use the input data to seed the array
	#
	if ($cacti_request == "get") {

		#
		# set snmp_arguments to memPoolindex plus the requested index value and query
		#
		$snmp_arguments[2] = $oid_array['memPoolindex'] . "." . $data_request_key;
		$snmp_test = trim(call_user_func_array("cacti_snmp_get", $snmp_arguments));

		#
		# the snmp response should contain an identifier for the MemPool component
		#
		if ((isset($snmp_test) == FALSE) ||
			(substr($snmp_test, 0, 16) == "No Such Instance") ||
			($snmp_test == "")) {

			echo ("FATAL: No MemPool data was returned from SNMP\n");
			return;
		}

		#
		# response looks okay, so assume the requested index value is valid
		#
		$mempool_array[0]['index'] = $data_request_key;
	}

	#
	# if they want data for all mempools, use snmpwalk to seed the array
	#
	else {
		#
		# set the snmp_arguments array to the mempool index OID
		#
		$snmp_arguments[2] = $oid_array['memPoolindex'];

		#
		# walk the tree and capture the resulting array of mempools
		#
		$snmp_array = call_user_func_array("cacti_snmp_walk", $snmp_arguments);

		#
		# verify that the response contains expected data structures
		#
		if ((isset($snmp_array) == FALSE) ||
			(count($snmp_array) == 0) ||
			(array_key_exists('oid', $snmp_array[0]) == FALSE) || 
			(array_key_exists('value', $snmp_array[0]) == FALSE) ||
			(substr($snmp_array[0]['value'],0,16) == "No Such Instance") ||
			(trim($snmp_array[0]['value']) == "")) {

			echo ("FATAL: No MemPool data was returned from SNMP\n");
			return;
		}

		#
		# create the array entries
		#
		$mempool_count = 0;

		foreach ($snmp_array as $snmp_response) {

			#
			# the last octet of the oid contains the MemPool index
			#
			$snmp_response['oid'] = trim($snmp_response['oid']);

			#
			# use regex to locate the relative OID
			#
			# exit if no match found
			#
			if (preg_match('/(\d+)$/', $snmp_response['oid'], $scratch) == 0) {

				echo ("FATAL: Unable to determine memPool index number from SNMP results\n");
				return;
			}

			else {
				#
				# match was found so use relative OID for the memPool index
				#
				$mempool_array[$mempool_count]['index'] = $scratch[1];
			}

			#
			# increment the mempool counter
			#
			$mempool_count++;
		}
	}

	#
	# verify that the mempool_array exists and has data
	#
	if ((isset($mempool_array) == FALSE) || (count($mempool_array) == 0)) {

		echo ("FATAL: No matching memPools were returned from SNMP\n");
		return;
	}

	#
	# requests for data other than index values require additional processing
	#
	if ((($cacti_request == "query") || ($cacti_request == "get")) &&
		($data_request != "memPool")) {

		#
		# cycle through the memPools and populate the data
		#
		$mempool_count = 0;

		foreach ($mempool_array as $mempool) {

			#
			# only fill in the requested data
			#
			# updates MUST reference the canonical array because foreach refs are just copies
			#

			switch ($data_request) {

				case "mempoolname":

					#
					# create a psuedo-name from the index value
					#
					$snmp_arguments[2] = $oid_array['memPoolindex'] . "." . $mempool['index'];
					$scratch = trim(call_user_func_array("cacti_snmp_get", $snmp_arguments));

                                        #
                                        # snmp response should contain the memPool name
                                        #
                                        if ((isset($scratch) == FALSE) ||
                                                (substr($scratch, 0, 16) == "No Such Instance") ||
                                                ($scratch == "")) {

                                                #
                                                # memPool name is unknown, so call it "Unnamed memPool"
                                                #
                                                $mempool_array[$mempool_count]['desc'] = "Unnamed memPool";
                                        }

					else {

						$mempool_array[$mempool_count]['name'] = $scratch . " Memory Pool";
					}

					break;

				case "mempoolused":

					#
					# fetch the 5min utilization for the current memPool
					#
					$snmp_arguments[2] = $oid_array['memPoolused'] . "." . $mempool['index'];
					$scratch = trim(call_user_func_array("cacti_snmp_get", $snmp_arguments));

					#
					# if no useful data was returned null the results
					#
					if ((isset($scratch) == FALSE) ||
						(substr($scratch, 0, 16) == "No Such Instance") ||
						(is_numeric($scratch) == FALSE) ||
						($scratch == "")) {

						$scratch = "nan";
					}

					$mempool_array[$mempool_count]['used'] = $scratch;

					break;
	
				case "mempooltotal":

					#
					# fetch the 5min available memory for the current memPool
					#
					$snmp_arguments[2] = $oid_array['memPoolfree'] . "." . $mempool['index'];
					$free = trim(call_user_func_array("cacti_snmp_get", $snmp_arguments));

					#
					# if no useful data was returned null the results
					#
					if ((isset($free) == FALSE) ||
						(substr($free, 0, 16) == "No Such Instance") ||
						(is_numeric($free) == FALSE) ||
						($free == "")) {

						$free = "";
					}

                                        #
                                        # fetch the 5min utilization for the current memPool
                                        #
                                        $snmp_arguments[2] = $oid_array['memPoolused'] . "." . $mempool['index'];
                                        $used = trim(call_user_func_array("cacti_snmp_get", $snmp_arguments));

                                        #
                                        # if no useful data was returned null the results
                                        #
                                        if ((isset($used) == FALSE) ||
                                                (substr($used, 0, 16) == "No Such Instance") ||
                                                (is_numeric($used) == FALSE) ||
                                                ($used == "")) {

                                                $used = "";
                                        }

					if ( (!empty($free)) && (!empty($used)) ) {
						$mempool_array[$mempool_count]['total'] = ($free + $used);
					} else {
						$mempool_array[$mempool_count]['total'] = "nan";
					}

					break;

			}

			#
			# increment the mempool counter
			#
			$mempool_count++;
		}
	}

	#
	# generate output
	#
	foreach ($mempool_array as $mempool) {

		#
		# return output data according to $cacti_request
		#
		switch ($cacti_request) {

			#
			# for "index" requests, dump the device column
			#
			case "index":

				echo ($mempool['index'] . "\n");
				break;

			#
			# if the user queried for all entries in a column, return that column
			#
			case "query";

				switch ($data_request) {

					case "mempool":

						echo ($mempool['index'] . ":" . $mempool['index'] . "\n");
						break;

					case "mempoolname":

						echo ($mempool['index'] . ":" . $mempool['name'] . "\n");
						break;

					case "mempoolused":

						echo ($mempool['index'] . ":" . $mempool['used'] . "\n");
						break;

					case "mempooltotal":

						echo ($mempool['index'] . ":" . $mempool['total'] . "\n");
						break;
				}

				break;

			#
			# if the user requested the value for a specific entry, give them just that data
			#
			case "get":

				switch ($data_request) {

					case "mempool":

						echo ($mempool['index'] . "\n");
						break;

					case "mempoolname":

						echo ($mempool['name'] . "\n");
						break;

					case "mempoolused":

						if (isset($GLOBALS['called_by_script_server']) == TRUE) {

							return($mempool['used']);
						}

						else {
							echo ($mempool['used'] . "\n");
						}

						break;

					case "mempooltotal":

						if (isset($GLOBALS['called_by_script_server']) == TRUE) {

							return($mempool['total']);
						}

						else {
							echo ($mempool['total'] . "\n");
						}

						break;
				}

				break;
		}
	}
}

#
# display the syntax
#
function ss_cisco_5min_mem_stats_syntax() {

	echo ("Syntax: ss_cisco_5min_mem_stats.php <hostname>:<snmp_version>:[<snmp_community>]:\ \n" .
	"      [<snmp3_username>]:[<snmp3_password>]:[<snmp3_auth_protocol>]:[<snmp3_priv_password>]:\ \n" .
	"      [<snmp3_priv_protocol>]:[<snmp3_context>]:[<snmp_port>]:[<snmp_timeout>] \ \n" .
	"      (index|query <fieldname>|get <fieldname> <mempool-index>)\n");
}

?>
